<?php

/**
 * creatic functions and definitions
 *
 * @link https://developer.wordpress.org/themes/basics/theme-functions/
 *
 * @package creatic
 * @subpackage Functions
 * @since 1.0
 */
 
if ( ! isset( $content_width ) )
	$content_width = 1070;

// Load Codeless Framework Constants
require_once( get_template_directory() . '/includes/core/codeless_constants.php' );

// Load Customizer Control Types
include_once( get_template_directory() . '/includes/codeless_customizer/kirki/kirki.php' ); 

// Load Required Plugins Tool
require_once( get_template_directory().'/includes/core/codeless_required_plugins.php' );
//Demo Importer
require_once( get_template_directory().'/includes/codeless_theme_panel/importer/codeless_importer.php' );
/**
 * All Start here...
 * 
 * @since 1.0.0
 */
function creatic_setup(){

    // Register Nav Menus Locations to use
    creatic_navigation_menus();
    // Load Codeless Customizer files and Options
    creatic_load_customizer();
    // Load All Codeless Framework Files
    creatic_load_framework();
    // Load Codeless Modules
    creatic_load_modules();

    // Language and text-domain setup
    add_action('init', 'creatic_language_setup');

    // Register Scripts and Styles
    add_action('wp_enqueue_scripts', 'creatic_register_global_styles');
    add_action('wp_enqueue_scripts', 'creatic_register_global_scripts');

    // Https filters
    add_filter( 'https_ssl_verify', '__return_false' );
    add_filter( 'https_local_ssl_verify', '__return_false' );

    // WP features that this theme supports
    creatic_theme_support();
    // Create Custom Image Sizes
    creatic_images_sizes(); 
    

    // Widgets
    creatic_load_widgets();
    creatic_register_widgets();  

    // Megamenu Create
    new codeless_custom_menu();
}

add_action( 'after_setup_theme', 'creatic_setup' );


/**
 * After theme activation
 * 
 * @since 1.0.0
 */
function creatic_after_switch_theme(){
    wp_redirect('admin.php?page=codeless-panel');
}

add_action( 'after_switch_theme', 'creatic_after_switch_theme' );


/**
 * Load Customizer Related Options and Customizer Cotrols Classes
 * 
 * @since 1.0.0
 */
function creatic_load_customizer() {

    // Load and Initialize Codeless Customizer
    include_once( get_template_directory() . '/includes/codeless_customizer/codeless_customizer_config.php' );
}


/**
 * Load all Codeless Framework Files
 * 
 * @since 1.0.0
 */
function creatic_load_framework() {

    // Register all Theme Hooks (add_action, add_filter)
    require_once( get_template_directory() . '/includes/codeless_hooks.php' );

    // Codeless Routing Templates and Custom Type Queries
    require_once( get_template_directory().'/includes/core/codeless_routing.php' );
    

    // Register all theme related sidebars
    require_once( get_template_directory().'/includes/register/register_sidebars.php' );

    // Register Custom Post Types
    // Works with Codeless Builder activated
    // Plugin Territory
    require_once( get_template_directory().'/includes/register/register_custom_types.php' );

    // Load Codeless Post Like
    require_once( get_template_directory().'/includes/core/codeless_post_like.php' );

    // Load Codeless Share Counts
    require_once( get_template_directory().'/includes/core/codeless_share_counts.php' );

    // Load Megamenu
    require_once( get_template_directory().'/includes/core/codeless_megamenu.php' );

    // Load all functions that are responsable for Extra Classes and Extra Attrs
    require_once( get_template_directory().'/includes/codeless_html_attrs.php' );

    // Load all blog related functions
    require_once( get_template_directory().'/includes/codeless_functions_blog.php' );

    // Load all portfolio related functions
    require_once( get_template_directory().'/includes/codeless_functions_portfolio.php' );

    // Load Theme Panels
    require_once( get_template_directory().'/includes/codeless_theme_panel/codeless_backpanel.php' );
    require_once( get_template_directory().'/includes/codeless_theme_panel/codeless_theme_panel.php' );
    require_once( get_template_directory().'/includes/codeless_theme_panel/codeless_image_sizes.php' );
    require_once( get_template_directory().'/includes/codeless_theme_panel/codeless_modules.php' ); 
    require_once( get_template_directory().'/includes/codeless_theme_panel/codeless_custom_sidebars.php' ); 
    require_once( get_template_directory().'/includes/codeless_theme_panel/codeless_system_status.php' );
    
    // Image Resize - Module - Resize image only when needed
    require_once( get_template_directory().'/includes/core/codeless_image_resize.php' );

    // Load Comment Walker
    require_once( get_template_directory().'/includes/core/codeless_comment_walker.php' );
    
    // Codeless Icons List
    require_once( get_template_directory().'/includes/core/codeless_icons.php' );

    // Fallback Class for Header when Codeless Builder Plugin is not active
    require_once( get_template_directory().'/includes/core/codeless_header_fallback.php' );

}



function creatic_load_builder_configs(){
    if( class_exists( 'Cl_Post_Meta' ) )
        require_once locate_template( 'includes/codeless_builder/config/cl-post-meta.php' );
}

add_action( 'init', 'creatic_load_builder_configs', 9 );


/**
 * Load All Modules
 * 
 * @since 1.0.0
 */
function creatic_load_modules(){
   require_once( get_template_directory().'/includes/codeless_modules/custom_portfolio_overlay_color.php' );
   require_once( get_template_directory().'/includes/codeless_modules/header_boxed_per_page.php' );
   require_once( get_template_directory().'/includes/codeless_modules/custom_primary_color_page.php' );
   require_once( get_template_directory().'/includes/codeless_modules/custom_header_background_per_page.php' );    
}


/**
 * Load Codeless Custom Widgets
 * 
 * @since 1.0.0
 */
function creatic_load_widgets() {
    require_once get_template_directory().'/includes/widgets/creatic_flickr.php';
    require_once get_template_directory().'/includes/widgets/creatic_mostpopular.php';
    require_once get_template_directory().'/includes/widgets/creatic_shortcodewidget.php';
    require_once get_template_directory().'/includes/widgets/creatic_socialwidget.php';
    require_once get_template_directory().'/includes/widgets/creatic_ads.php';
    require_once get_template_directory().'/includes/widgets/creatic_service.php';
    require_once get_template_directory().'/includes/widgets/creatic_contactinfo.php';
    require_once get_template_directory().'/includes/widgets/creatic_button.php';
    require_once get_template_directory().'/includes/widgets/creatic_instagram.php';
}


/**
 * Setup Language Directory and theme text domain
 * 
 * @since 1.0.0
 */
function creatic_language_setup() {
    $lang_dir = get_template_directory() . '/lang';
    load_theme_textdomain('creatic', $lang_dir);
} 


/**
 * Add Theme Supports
 * 
 * @since 1.0.0
 */
function creatic_theme_support(){
    add_theme_support( 'post-thumbnails' );

    add_theme_support( 'automatic-feed-links' );
    add_theme_support('nav_menus');
    add_theme_support( 'post-formats', array( 'quote', 'gallery','video', 'audio', 'link' ) ); 
    add_theme_support( "title-tag" );
    add_theme_support( 'html5', array( 'comment-list', 'comment-form', 'search-form', 'gallery', 'caption' ) );

    // Add theme support for selective refresh for widgets.
	add_theme_support( 'customize-selective-refresh-widgets' );
    add_editor_style();
}


/**
 * Regster Theme Related Image Sizes
 * 
 * @since 1.0.0
 */
function creatic_images_sizes(){
    // Empty
}


/**
 * Register Navigation Menus
 * 
 * @since 1.0.0
 */
function creatic_navigation_menus(){
    $navigations = array('main' => esc_attr__('Main Navigation', 'creatic') );

    foreach($navigations as $id => $name){ 
    	register_nav_menu($id, THEMETITLE.' '.$name); 
    }
}


/**
 * Regster Loaded Widgets
 * 
 * @since 1.0.0
 */
function creatic_register_widgets(){
    register_widget( 'CreaticSocialWidget' );
    register_widget( 'CreaticFlickrWidget' );
    register_widget( 'CreaticShortcodeWidget' );
    register_widget( 'CreaticMostPopularWidget');
    register_widget( 'CreaticAdsWidget');
    register_widget( 'CreaticService');
    register_widget( 'CreaticContactInfo');
    register_widget( 'CreaticButton' );
    register_widget( 'CreaticInstagram' );

   
}


/**
 * Enqueue all needed styles
 * 
 * @since 1.0.0
 */
function creatic_register_global_styles(){ 

    wp_enqueue_style('bootstrap', get_template_directory_uri() . '/css/bootstrap.css');
    wp_enqueue_style('creatic-style', get_stylesheet_uri());
    
    wp_enqueue_style('creatic-front-elements', get_template_directory_uri() . '/css/creatic-front-elements.css');

    wp_enqueue_style('swiper-slider', get_template_directory_uri() . '/css/swiper.min.css');
    
    if( codeless_get_mod( 'codeless_page_transition' )) 
        wp_enqueue_style('animsition', get_template_directory_uri(). '/css/animsition.min.css'); 
    
    if( codeless_get_mod( 'blog_entry_overlay_icon' ) == 'lightbox' )
        wp_enqueue_style('ilightbox', get_template_directory_uri() . '/css/ilightbox/css/ilightbox.css' );
    
    wp_enqueue_style('select2', get_template_directory_uri() . '/css/select2.min.css');

    wp_enqueue_style('owl-carousel', get_template_directory_uri() .'/css/owl.carousel.min.css' ); 
    // Create Dynamic Styles
    wp_enqueue_style( 'creatic-dynamic', get_template_directory_uri() . '/css/creatic-dynamic.css');
    
    /* Load Custom Dynamic Style and enqueue them with wp_add_inline_style */
    ob_start();
    creatic_custom_dynamic_style();
    $styles = ob_get_clean();

    wp_add_inline_style( 'creatic-dynamic', apply_filters( 'creatic_register_styles', $styles ) );    
}


/**
 * Enqueue all global scripts
 * 
 * @since 1.0.0
 */
function creatic_register_global_scripts(){
    
    wp_enqueue_script( 'creatic-main', get_template_directory_uri() . '/js/creatic-main.js', array( 'jquery', 'imagesloaded' ) );
    wp_enqueue_script( 'imagesloaded' );
    wp_enqueue_script( 'bowser', get_template_directory_uri() . '/js/bowser.min.js' );

   if( codeless_get_mod( 'codeless_page_transition', false )) 
        wp_enqueue_script('animation', get_template_directory_uri(). '/js/animsition.min.js'); 

    if( codeless_get_mod( 'nicescroll' ) )
        wp_enqueue_script( 'smoothscroll', get_template_directory_uri() . '/js/smoothscroll.js' ); 

    if( is_singular() && comments_open() && ( get_option( 'thread_comments' ) == 1) ) {
        // Load comment-reply.js
        wp_enqueue_script( 'comment-reply' );
    }



    wp_enqueue_style('creatic-creative-search', get_template_directory_uri() . '/css/creatic-creative-search.css');

    wp_localize_script(
        'creatic-main',
        'creatic_global',
        array(
            'ajax_url' => esc_url(admin_url( 'admin-ajax.php' )),
            'FRONT_LIB_JS' => esc_url( get_template_directory_uri() . '/js/' ) ,
            'FRONT_LIB_CSS' => esc_url( get_template_directory_uri() . '/css/' ),
            'postSwiperOptions' => codeless_get_post_slider_options(),
            'cl_btn_classes' => esc_attr( codeless_button_classes() ),
            'wc_placeholder_img_src' => ( function_exists('wc_placeholder_img_src') ? esc_url( wc_placeholder_img_src() ) : '' ),
            'shop_columns_mobile' => esc_attr( codeless_get_mod( 'shop_columns_mobile', 1 ) ),
            'shop_open_toggles' => esc_attr( codeless_get_mod( 'shop_open_toggles', 0 ) ),
            'language' => array(
                'added' => esc_attr__('Added', 'creatic'),
                'add_to_cart' => esc_attr__('Add to Cart', 'creatic')
            )
        )
    );
}

/**
 * Load all styles from register_styles.php
 * Added with wp_add_inline_style on codeless_register_global_styles, action wp_enqueue_scripts
 * @since 1.0.0
 */
function creatic_custom_dynamic_style(){
    include get_template_directory().'/includes/register/register_styles.php';
}

require_once( get_template_directory().'/functions-codeless.php' );

?>